@@ -652,6 +652,11 @@ static void plugin_call_watches(PluginHandle * plugin)
     plugin->watches.remove_if(call_and_check_remove);
 }
 
+int plugin_get_priority(PluginHandle * plugin)
+{
+    return plugin->priority;
+}
+
 PluginEnabled plugin_get_enabled(PluginHandle * plugin)
 {
     return plugin->enabled;
@@ -754,7 +759,7 @@ EXPORT Index<const char *> aud_plugin_get_supported_mime_types()
         if (!aud_plugin_get_enabled(p))
             continue;
 
-        for (auto k : p->keys[InputKey::MIME])
+        for (auto & k : p->keys[InputKey::MIME])
             mimes.append((const char *)k);
     }
 
