@@ -148,6 +148,9 @@ static const struct
     {"description", Tuple::String, -1},
     {"musicbrainz-id", Tuple::String, -1},
     {"channels", Tuple::Int, -1},
+    {"publisher", Tuple::String, -1},
+    {"catalog-number", Tuple::String, -1},
+    {"lyrics", Tuple::String, -1},
 
     /* fallbacks */
     {nullptr, Tuple::String, -1},
@@ -171,6 +174,7 @@ static const FieldDictEntry field_dict[] = {
     {"artist", Tuple::Artist},
     {"audio-file", Tuple::AudioFile},
     {"bitrate", Tuple::Bitrate},
+    {"catalog-number", Tuple::CatalogNum},
     {"channels", Tuple::Channels},
     {"codec", Tuple::Codec},
     {"comment", Tuple::Comment},
@@ -190,8 +194,10 @@ static const FieldDictEntry field_dict[] = {
     {"gain-track-peak", Tuple::TrackPeak},
     {"genre", Tuple::Genre},
     {"length", Tuple::Length},
+    {"lyrics", Tuple::Lyrics},
     {"musicbrainz-id", Tuple::MusicBrainzID},
     {"performer", Tuple::Performer},
+    {"publisher", Tuple::Publisher},
     {"quality", Tuple::Quality},
     {"segment-end", Tuple::EndTime},
     {"segment-start", Tuple::StartTime},
