@@ -64,7 +64,7 @@ static void finish_job (void * data)
     ImportExportJob * job = (ImportExportJob *) data;
 
     Playlist::GetMode mode = Playlist::Wait;
-    if (aud_get_bool (nullptr, "metadata_on_play"))
+    if (aud_get_bool ("metadata_on_play"))
         mode = Playlist::NoWait;
 
     if (job->list.exists ())
